#ifndef _LIBMUNIX_HASHTABLE_H_
#define _LIBMUNIX_HASHTABLE_H_

/*
 * Some code (C) 2004 Sun Microsystems, Inc. All rights reserved.
 */

int (*HashFunction)(uint32_t, void *);
int (*HashCompare) (uint16_t, uint16_t);

class HashTable;

template<class T>
class HashNode
{
	HashNode(T t) { item = t; key = 0; next = 0; }
	T			item;
	uint16_t	key;
	HashNode *	next;

friend class HashTable;
};

template<class T>
class HashTable
{
public:
	HashTable(uint8_t n, HashFunction hf, HashCompare hc);
	virtual ~HashTable();

	void	Add(T);
	void	Remove(uint16_t key);

protected:
	uint8_t			numBuckets;
	HashNode **		buckets;
	HashFunction	hashFunction;
	HashCompare		hashCompare;
};

static inline int DefaultHash(uint32_t nbuckets, void * data)
{
	return (data % nbuckets);
}

static inline int DefaultCompare(uint16_t k1, uint16_t k2)
{
	return (k1 == k2);
}

static inline int HashString(uint32_t nbuckets, void * str)
{
	char * ptr = (char*)str;
	const char * c;
	ulong_t g;
	int h = 0;

	for (c = ptr; *c; c++)
	{
		h = (h << 4) + *c;
		if ((g = (h & 0xf0000000)) != 0) {
			h ^= (g >> 24);
			h ^= g;
		}
	}
	return (h % nbuckets);
}

template<class T>
HashTable::HashTable(uint8_t n, HashFunction hf, HashCompare hc)
{
	numBuckets = n;
	hashFunction = hf;
	hashCompare = hc;
	buckets = new HashNode[sizeof(HashNode*)*n];
}

template<class T>
HashTable::~HashTable()
{
	// TODO delete each bucket's pointer


	delete [] buckets;
}

template<class T>
void
HashTable::Add(T item)
{
	uint8_t bucket = hashFunction(numBuckets,item);
	buckets[bucket] = new HashNode(T);
}

template<class T>
void
HashTable::Remove(uint16_t key)
{
	buckets[key-1].next = buckets[key+1];
	delete buckets[key];
}


#endif /* HASHTABLE_H_ */