/*******************************************************************************
 ** Name: types.h                                                             **
 ** Description:              **
 **                                                                           **
 ** Open Source Initiative (OSI) Approved License                             **
 **                                                                           **
 ** The contents of this file are subject to the terms of the                 **
 ** Common Development and Distribution License, Version 1.0 only             **
 ** (the "License").  You may not use this file except in compliance          **
 ** with the License.                                                         **
 **                                                                           **
 ** You can find a copy of the license in the license.txt within              **
 ** this distribution or at http://www.munixos.net/licensing.                 **
 ** Software distributed under the License is distributed on an "AS IS"       **
 ** basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.           **
 ** See the License for the specific language governing permissions           **
 ** and limitations under the License.                                        **
 **                                                                           **
 ** When distributing Covered Code, include this CDDL header in each          **
 ** file and include the License file at $HOME/license.txt.                   **
 ** If applicable, add the following below this header, with the indicated    **
 ** fields enclosed by brackets "[]" replaced with your own identifying       **
 ** information: Portions Copyright [yyyy] [name of copyright owner]          **
 **                                                                           **
 **                                                                           **
 ** Copyright (c) 2009-2010  Barry Gian James <bjames@munixos.net>            **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Ref: $HeadURL: https://DroidEngineer@munix.svn.codeplex.com/svn/trunk/libmunix/rbuff.cpp $
 ******************************************************************************/
// Portions (C) 2011 OpenHouseware, LLC.
// All Rights Reserved

// $Id: rbuff.cpp 56499 2011-03-14 15:25:03Z unknown $
// Last Modified $DateTime$ by $Author: unknown $
#include "rbuff.h"
#include <string.h>

void
RingBuff::CopyBuff(char * dest) const
{
	memcpy(dest, (char*)_buff, DEFAULT_RING_BUFF_SZ);
}


// Push onto the tail
void
RingBuff::Put(unsigned char c) volatile
{
	_buff[_tail++] = c;
	if (_tail >= _sz) _tail = 0;
	++_len;
}

// Pop from the head
unsigned char
RingBuff::Get() volatile
{
	if (_len == 0)
		return 0;

	unsigned char c = _buff[_head++];
	if (_head >= _sz) _head = 0;
	--_len;

	return c;
}

RingBuff::RingBuff(const RingBuff& rb)
{
    delete [] _buff;
    _buff = new volatile uchar[rb.Size()];
    rb.CopyBuff((char*)_buff);
    _head = rb.Head();
    _tail = rb.Tail();
    _sz   = rb.Size();
}

RingBuff::RingBuff(uint8 sz)
{
    _sz = sz;
    _buff = new volatile uchar[_sz];
	_head = _tail = _len = 0;
}
