#ifndef _MUNIX_RBUFF_H_
#define _MUNIX_RBUFF_H_
/*******************************************************************************
 ** Name: types.h                                                             **
 ** Description:              **
 **                                                                           **
 ** Open Source Initiative (OSI) Approved License                             **
 **                                                                           **
 ** The contents of this file are subject to the terms of the                 **
 ** Common Development and Distribution License, Version 1.0 only             **
 ** (the "License").  You may not use this file except in compliance          **
 ** with the License.                                                         **
 **                                                                           **
 ** You can find a copy of the license in the license.txt within              **
 ** this distribution or at http://www.munixos.net/licensing.                 **
 ** Software distributed under the License is distributed on an "AS IS"       **
 ** basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.           **
 ** See the License for the specific language governing permissions           **
 ** and limitations under the License.                                        **
 **                                                                           **
 ** When distributing Covered Code, include this CDDL header in each          **
 ** file and include the License file at $HOME/license.txt.                   **
 ** If applicable, add the following below this header, with the indicated    **
 ** fields enclosed by brackets "[]" replaced with your own identifying       **
 ** information: Portions Copyright [yyyy] [name of copyright owner]          **
 **                                                                           **
 **                                                                           **
 ** Copyright (c) 2009-2010  Barry Gian James <bjames@munixos.net>            **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Ref: $HeadURL: https://DroidEngineer@munix.svn.codeplex.com/svn/trunk/libmunix/rbuff.h $
 ******************************************************************************/
// Portions (C) 2011 OpenHouseware, LLC.
// All Rights Reserved

// $Id: rbuff.h 56499 2011-03-14 15:25:03Z unknown $
// Last Modified $DateTime$ by $Author: unknown $

#include "common.h"
#include <string.h>


#define DEFAULT_RING_BUFF_SZ	16	// power of 2!!

//! @class RingBuff
//! @brief Implements a queue or circular buffer
class RingBuff
{
public:
	RingBuff(uint8 sz);
        RingBuff(const RingBuff &);
	~RingBuff() { delete [] _buff;}

	void		Put(unsigned char) volatile;
	unsigned char	Get() volatile;
	inline uint8	Empty() const { return (_head == _tail); }
	inline uint8	Size() const { return _sz; }
	inline uint8	Length() const { return _len; }
	void		CopyBuff(char * dest) const;
	inline char *	Buff() { return (char*)_buff; }
	inline void	Clear() { memset((char*)_buff,0,DEFAULT_RING_BUFF_SZ); _head=_tail=0; }
        inline volatile uint8 Head() const { return _head; }
        inline volatile uint8 Tail() const { return _tail; }

private:
	volatile uchar *_buff;
	uint8		_sz;
	volatile uint8	_head;
	volatile uint8	_tail;
	volatile uint8	_len;
};




#endif	/* _MUNIX_RBUFF_H_ */


